#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <algorithm>
#include <cmath>
#include <vector>
#include <set>
#include <map>
#include <unordered_set>
#include <unordered_map>
#include <queue>
#include <ctime>
#include <cassert>
#include <complex>
#include <string>
#include <cstring>
#include <chrono>
#include <random>
#include <bitset>
using namespace std;

#ifdef LOCAL
	#define eprintf(...) fprintf(stderr, __VA_ARGS__);fflush(stderr);
#else
	#define eprintf(...) 42
#endif

using ll = long long;
using ld = long double;
using uint = unsigned int;
using ull = unsigned long long;
template<typename T>
using pair2 = pair<T, T>;
using pii = pair<int, int>;
using pli = pair<ll, int>;
using pll = pair<ll, ll>;
mt19937_64 rng(chrono::steady_clock::now().time_since_epoch().count());
ll myRand(ll B) {
	return (ull)rng() % B;
}

#define pb push_back
#define mp make_pair
#define all(x) (x).begin(),(x).end()
#define fi first
#define se second

clock_t startTime;
double getCurrentTime() {
	return (double)(clock() - startTime) / CLOCKS_PER_SEC;
}


struct Edmonds {
	vector<vector<int>> g;
	int n;
	vector<int> mt;
	vector<int> used;
	vector<int> par;
	vector<int> dsu;
	vector<int> tm;
	vector<int> Q;
	int T;

	Edmonds() : g(), n(0), mt(), used(), par(), dsu(), tm(), T() {}
	Edmonds(int _n) {
		n = _n;
		g.clear();
		g.resize(n);
		mt = vector<int>(n, -1);
		used = vector<int>(n, 0);
		par = vector<int>(n, -1);
		dsu = vector<int>(n, -1);
		tm = vector<int>(n, 0);
		T = 1;
	}

	void addEdge(int v, int u) {
		g[v].push_back(u);
		g[u].push_back(v);
	}

	void clear() {
		mt = vector<int>(n, -1);
	}

	int blossom(int v) {
		return (dsu[v] == -1 ? v : dsu[v] = blossom(dsu[v]));
	}
	void unite(int v, int u) {
		v = blossom(v);
		u = blossom(u);
		if (v == u) return;
		dsu[v] = u;
	}

	int lca(int v, int u) {
		T++;
		while(true) {
			if (v == -1 && u == -1) return -1;
			swap(v, u);
			if (v == -1) continue;
			v = blossom(v);
			if (tm[v] == T) return v;
			tm[v] = T;
			if (mt[v] != -1)
				v = par[mt[v]];
			else
				v = -1;
		}
	}

	void shrinkBlossom(int v, int p) {
		while(v != p) {
			int u = mt[v];
			int w = par[u];
			if (blossom(w) != p) par[w] = u;
			if (used[u] == 2) {
				used[u] = 1;
				Q.push_back(u);
			}
			if (used[w] == 2) {
				used[w] = 1;
				Q.push_back(w);
			}
			unite(v, u);
			unite(v, w);
			v = w;
		}
	}

	void alternateUp(int u) {
		while(u != -1) {
			int v = par[u];
			int w = mt[v];
			mt[v] = u;
			mt[u] = v;
			u = w;
		}
	}

	int BFS() {
		for (int it = 0; it < (int)Q.size(); it++) {
			int v = Q[it];
			for (int u : g[v]) {
				if (mt[v] == u) continue;
				if (blossom(v) == blossom(u)) continue;
				if (used[u] == 2) continue;
				if (used[u] == 1) {
					int base = lca(v, u);
					if (base == -1) {
						int pv = mt[v], pu = mt[u];
						mt[v] = u;
						mt[u] = v;
						alternateUp(pv);
						alternateUp(pu);
						return 1;
					}
					if (blossom(v) != base) par[v] = u;
					if (blossom(u) != base) par[u] = v;
					shrinkBlossom(v, base);
					shrinkBlossom(u, base);
				} else if (mt[u] == -1) {
					par[u] = v;
					alternateUp(u);
					return 1;
				} else {
					par[u] = v;
					used[u] = 2;
					u = mt[u];
					used[u] = 1;
					Q.push_back(u);
				}
			}
		}
		return 0;
	}

	int tryIncrease() {
		Q.clear();
		for (int i = 0; i < n; i++) {
			used[i] = 0;
			par[i] = -1;
			dsu[i] = -1;
		}
		int RES = 0;
		for (int v = 0; v < n; v++) {
			if (mt[v] != -1) continue;
			bool fnd = false;
			for (int u : g[v]) {
				if (mt[u] == -1) {
					fnd = true;
					RES++;
					mt[v] = u;
					mt[u] = v;
					break;
				}
			}
			if (fnd) continue;
			used[v] = 1;
			Q.push_back(v);
		}
		return RES + BFS();
	}

	int increaseMatching() {
		int res = 0;
		while(true) {
			int cur = tryIncrease();
			if (cur == 0) break;
			res += cur;
		}
		return res;
	}
};

const int N = 505;
const int M = N * N;
int n, m, k;
int ed[M][2];
bool g[N][N];
int id[N];
int ord[N];
int ordSz;
int col[N];
bool used[N];
int C;
int ans[N];

void dfsOrd(int v) {
	used[v] = 1;
	for (int u = 0; u < 2 * k; u++) {
		if (!g[u][v]) continue;
		if (used[u]) continue;
		dfsOrd(u);
	}
	ord[ordSz++] = v;
}
void dfsCol(int v) {
	col[v] = C;
	for (int u = 0; u < 2 * k; u++) {
		if (!g[v][u]) continue;
		if (col[u] != -1) continue;
		dfsCol(u);
	}
}
void dfsAns(int v) {
	if (ans[v / 2] != -1) return;
	ans[v / 2] = v & 1;
	for (int u = 0; u < 2 * k; u++) {
		if (!g[v][u]) continue;
		dfsAns(u);
	}
}

vector<int> solve(vector<pii> mt, vector<int> taken) {
	for (int i = 0; i < n; i++)
		id[i] = -1;
	k = (int)mt.size();
	for (int i = 0; i < k; i++) {
		id[mt[i].first] = 2 * i;
		id[mt[i].second] = 2 * i + 1;
	}
	for (int v : taken)
		id[v] = -2;
	for (int i = 0; i < 2 * k; i++)
		for (int j = 0; j < 2 * k; j++)
			g[i][j] = 0;
	for (int i = 0; i < m; i++) {
		int v = id[ed[i][0]], u = id[ed[i][1]];
		if (v == -2 || u == -2) continue;
		if (v == -1) swap(v, u);
		if (u == -1) {
			g[v ^ 1][v] = 1;
		} else {
			g[v ^ 1][u] = 1;
			g[u ^ 1][v] = 1;
		}
	}
	ordSz = 0;
	for (int i = 0; i < n; i++) {
		used[i] = 0;
		col[i] = -1;
	}
	for (int v = 0; v < 2 * k; v++) {
		if (used[v]) continue;
		dfsOrd(v);
	}
	reverse(ord, ord + ordSz);
	C = 0;
	for (int i = 0; i < 2 * k; i++) {
		int v = ord[i];
		if (col[v] != -1) continue;
		dfsCol(v);
		C++;
	}
	for (int i = 0; i < k; i++) {
		if (col[2 * i] == col[2 * i + 1])
			return {};
	}
	for (int i = 0; i < k; i++)
		ans[i] = -1;
	for (int i = 0; i < 2 * k; i++) {
		int v = ord[i];
		if (ans[v / 2] != -1) continue;
		dfsAns(v);
	}
	for (int i = 0; i < k; i++) {
		if (ans[i] == 0)
			taken.push_back(mt[i].first);
		else
			taken.push_back(mt[i].second);
	}
	return taken;
}

int main()
{
	startTime = clock();
//	freopen("seating.in", "r", stdin);
//	freopen("seating.out", "w", stdout);

	scanf("%d%d", &n, &m);
	if (m == 0) {
		printf("0\n\n");
		return 0;
	}
	Edmonds E = Edmonds(n);
	for (int i = 0; i < m; i++) {
		for (int j = 0; j < 2; j++) {
			scanf("%d", &ed[i][j]);
			ed[i][j]--;
		}
		E.addEdge(ed[i][0], ed[i][1]);
	}
	E.increaseMatching();
	vector<pii> mt;
	for (int v = 0; v < n; v++) {
		if (E.mt[v] > v)
			mt.push_back(mp(v, E.mt[v]));
	}
	vector<int> res = {};
	res = solve(mt, {});
	if (!res.empty()) {
		printf("%d\n", (int)res.size());
		sort(all(res));
		for (int v : res)
			printf("%d ", v + 1);
		printf("\n");
		return 0;
	}
	for (int v = 0; v < n; v++) {
		if (E.mt[v] != -1) continue;
		res = solve(mt, {v});
		if (!res.empty()) {
			printf("%d\n", (int)res.size());
			sort(all(res));
			for (int v : res)
				printf("%d ", v + 1);
			printf("\n");
			return 0;
		}
	}
	for (int i = 0; i < (int)mt.size(); i++) {
		vector<pii> mt2 = mt;
		mt2.erase(mt2.begin() + i);
		res = solve(mt2, {mt[i].first, mt[i].second});
		if (!res.empty()) {
			printf("%d\n", (int)res.size());
			sort(all(res));
			for (int v : res)
				printf("%d ", v + 1);
			printf("\n");
			return 0;
		}
	}
	printf("not smol\n");

	return 0;
}
