#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <algorithm>
#include <cmath>
#include <vector>
#include <set>
#include <map>
#include <unordered_set>
#include <unordered_map>
#include <queue>
#include <ctime>
#include <cassert>
#include <complex>
#include <string>
#include <cstring>
#include <chrono>
#include <random>
#include <bitset>
using namespace std;

#ifdef LOCAL
	#define eprintf(...) fprintf(stderr, __VA_ARGS__);fflush(stderr);
#else
	#define eprintf(...) 42
#endif

using ll = long long;
using ld = long double;
using uint = unsigned int;
using ull = unsigned long long;
template<typename T>
using pair2 = pair<T, T>;
using pii = pair<int, int>;
using pli = pair<ll, int>;
using pll = pair<ll, ll>;
mt19937_64 rng(chrono::steady_clock::now().time_since_epoch().count());
ll myRand(ll B) {
	return (ull)rng() % B;
}

#define pb push_back
#define mp make_pair
#define all(x) (x).begin(),(x).end()
#define fi first
#define se second

clock_t startTime;
double getCurrentTime() {
	return (double)(clock() - startTime) / CLOCKS_PER_SEC;
}

struct Fenwick {
	vector<int> fenv;

	Fenwick() : fenv() {}
	Fenwick(int n) : fenv(vector<int>(n, 0)) {}

	void add(int p, int x) {
		for(; p < (int)fenv.size(); p |= p + 1)
			fenv[p] += x;
	}
	int get(int r) {
		int res = 0;
		for(; r >= 0; r = (r & (r + 1)) - 1)
			res += fenv[r];
		return res;
	}
	int getSum(int l, int r) {
		if (l >= r) return 0;
		return get(r - 1) - get(l - 1);
	}
} fenv;


const int N = 500500;
const int SZ = 1 << 19;
struct Node {
	int l, r;
	pii val;
	int toAdd;

	Node() : l(), r(), val(), toAdd() {}
	Node(int _l, int _r) : l(_l), r(_r), val(mp(0, _l)), toAdd(0) {}

	void add(int x) {
		val.first += x;
		toAdd += x;
	}
};
Node tree[2][2 * SZ + 5];
int L;

void build(int n) {
	L = max(n + 1, 2);
	while(L & (L - 1)) L++;
	for (int k = 0; k < 2; k++) {
		for (int i = 0; i < L; i++)
			tree[k][L + i] = Node(i, i + 1);
		for (int i = L - 1; i > 0; i--)
			tree[k][i] = Node(tree[k][2 * i].l, tree[k][2 * i + 1].r);
	}
}
void push(int k, int v) {
	assert(v < L);
	for (int u = 2 * v; u < 2 * v + 2; u++) {
		tree[k][u].add(tree[k][v].toAdd);
	}
	tree[k][v].toAdd = 0;
}
void update(int k, int v) {
	assert(v < L);
	tree[k][v].val = max(tree[k][2 * v].val, tree[k][2 * v + 1].val);
}

void addOnSegm(int k, int v, int l, int r, int x) {
	if (l <= tree[k][v].l && tree[k][v].r <= r) {
		tree[k][v].add(x);
		return;
	}
	if (l >= tree[k][v].r || tree[k][v].l >= r) return;
	push(k, v);
	for (int u = 2 * v; u < 2 * v + 2; u++)
		addOnSegm(k, u, l, r, x);
	update(k, v);
}
pii getMax(int k, int v, int l, int r) {
	if (l <= tree[k][v].l && tree[k][v].r <= r)
		return tree[k][v].val;
	if (l >= tree[k][v].r || tree[k][v].l >= r)
		return mp(-(10 * N), -(10 * N));
	push(k, v);
	return max(getMax(k, 2 * v, l, r), getMax(k, 2 * v + 1, l, r));
}

int n, q;
int a[N];
int ANS[N];

struct Query {
	int l, r, id;

	Query() : l(), r(), id() {}
	Query(int _l, int _r) : l(_l), r(_r), id(-1) {}

	void read(int _id) {
		id = _id;
		scanf("%d%d", &l, &r);
		l--;
	}

	bool operator < (const Query &Q) const {
		if (l != Q.l) return l < Q.l;
		return r < Q.r;
	}
};
Query b[N];
vector<int> qu[N];

int main()
{
	startTime = clock();
//	freopen("seating.in", "r", stdin);
//	freopen("seating.out", "w", stdout);

	scanf("%d%d", &n, &q);
	for (int i = 0; i < n; i++) {
		scanf("%d", &a[i]);
		a[i] = min(a[i], n);
	}
	
	build(n + 1);
	fenv = Fenwick(n);
	for (int i = 0; i < n; i++) {
		fenv.add(i, 1);
		qu[a[i]].push_back(i);
	}

	for (int i = 0; i < q; i++)
		b[i].read(i);
	sort(b, b + q);
	set<pii> setik;
	setik.insert(mp(-1, -1));
	for (int i = 0; i < q; i++) {
		addOnSegm(1, 1, i, i + 1, b[i].r);
		if (setik.rbegin()->first < b[i].r) {
			eprintf("init active %d(%d)\n", i, b[i].id);
			addOnSegm(0, 1, i, i + 1, b[i].r - b[i].l);
			setik.insert(mp(b[i].r, i));
		} else {
			addOnSegm(0, 1, i, i + 1, -N);
		}
	}
	setik.insert(mp(n + 1, n));
	for (int x = n; x > 0; x--) {
		for (int p : qu[x]) {
			fenv.add(p, -1);
			int r = lower_bound(b, b + q, Query(p + 1, -1)) - b;
			auto it = setik.lower_bound(mp(p + 1, -1));
			assert(it != setik.end());
			int l = it->second;
			if (l < r)
				addOnSegm(0, 1, l, r, -1);
		}
		while(true) {
			pii w = tree[0][1].val;
			eprintf("%d -- (%d %d)\n", x, w.first, w.second);
			if (w.first < x) break;
			int p = w.second;
			eprintf("find ans for %d(%d)\n", p, b[p].id);
			ANS[b[p].id] = x;
			auto it = setik.lower_bound(mp(b[p].r, p));
			it++;
			int R = it->second;
			it--;
			it = setik.erase(it);
			assert(it != setik.begin());
			it--;
			int W = it->first;
			addOnSegm(0, 1, p, p + 1, -(2 * N));
			addOnSegm(1, 1, p, p + 1, -(2 * N));
			while(true) {
				pii cur = getMax(1, 1, 0, R);
				if (cur.first <= W) break;
				R = cur.second;
				int nval = fenv.getSum(b[R].l, b[R].r);
				eprintf("activate %d(%d) -- %d\n", R, b[R].id, nval);
				addOnSegm(0, 1, R, R + 1, nval - getMax(0, 1, R, R + 1).first);
				eprintf("val after activation = %d\n", getMax(0, 1, R, R + 1).first);
				setik.insert(mp(b[R].r, R));
			}
		}
	}

	for (int i = 0; i < q; i++)
		printf("%d\n", ANS[i]);

	return 0;
}
