#ifndef LOCAL
//#pragma GCC optimize("Ofast")
//#pragma GCC target("sse,sse2,sse3,ssse3,sse4,popcnt,abm,mmx,avx,avx2,fma,tune=native")
#endif

#include <iostream>
#include <vector>
#include <cstdio>
#include <algorithm>
#include <set>
#include <map>
#include <cassert>
#include <numeric>
#include <string>
#include <cstring>
#include <cmath>
#include <queue>
#include <random>
#include <unordered_set>
#include <queue>
#include <ctime>
#include <chrono>
using namespace std;

#ifdef LOCAL
	#define eprintf(...) fprintf(stderr, __VA_ARGS__)
#else
	#define eprintf(...) 42
#endif


typedef long long int int64;
typedef long long int ll;
typedef long long unsigned ull;

mt19937_64 rng(chrono::steady_clock::now().time_since_epoch().count());
ll myRand(ll B) {
	return (ull)rng() % B;
}

const int LOG = 18;
const int LEVEL = (1 << LOG);
const int SZ = (LEVEL << 1);
const int N = (int) 2e5 + 100;

struct Node {

    unordered_set <int> grey;
    priority_queue <int> white;
    int ans;

    Node() : grey(), white(), ans(N) {}

    void setWhiteIfNew(int level) {
        if (!grey.count(level)) {
            // used.insert(level);
            white.push(-level);
        }
    }

    void setBlack(int level) {
        grey.insert(level);
    }
    
    int updateAns(int ansL, int ansR) {
        int x = max(ansL, ansR);
        while (!white.empty() && grey.count(-white.top()))
            white.pop();
        if (!white.empty())
            x = min(x, -white.top());
        return x;
    }
};

struct Tree {
    Node node[SZ];

    Tree() : node() {}

    void update(int v) {
        if (v < LEVEL)
            node[v].ans = node[v].updateAns(node[2 * v].ans, node[2 * v + 1].ans);
        else
            node[v].ans = node[v].updateAns(N, N);
    }

    void addSegment(int v, int tl, int tr, int l, int r, int y) {
        if (r < tl || tr < l) {
            node[v].setWhiteIfNew(y);
            update(v);
            return;
        }
        if (l <= tl && tr <= r) {
            node[v].setBlack(y);
            update(v);
            return;
        }
        node[v].setBlack(y); // it's Grey in fact. But the methods are equivalent.
        int tm = (tl + tr) / 2;
        addSegment(2 * v, tl, tm, l, r, y);
        addSegment(2 * v + 1, tm + 1, tr, l, r, y);
        update(v);
    }

    void addSegment(int l, int r, int y) {
        addSegment(1, 0, LEVEL - 1, l, r, y);
    }

    int getAns(int v, int tl, int tr, int l, int r) {
        if (r < tl || tr < l)
            return 0;
        if (l <= tl && tr <= r) {
            return node[v].ans;
        }
        int tm = (tl + tr) / 2;
        int ansL = getAns(2 * v, tl, tm, l, r);
        int ansR = getAns(2 * v + 1, tm + 1, tr, l, r);
        return node[v].updateAns(ansL, ansR);
    }

    int getAns(int l, int r) {
        return getAns(1, 0, LEVEL - 1, l, r);
    }
} tree;


set <pair <int, int>> segments[N];

void addSimpleSegment(int y, int l, int r) {
    tree.addSegment(l, r, y);
}

void addSegment(int y, int l, int r) {
    auto it = segments[y].lower_bound(make_pair(l, r));
    if (it != segments[y].begin()) {
        it--;
    }
    vector <pair <int, int>> todel;
    int newl = l, newr = r;

    while (l <= r && it != segments[y].end()) {
        if (it->second < l) {
            it++;
            continue;
        }
        if (it->first > r) {
            break;
        }
        newl = min(newl, it->first);
        newr = max(newr, it->second);
        todel.push_back(*it);

        if (l < it->first)
            addSimpleSegment(y, l, it->first - 1);
        l = it->second + 1;
        it++;
    }
    if (l <= r)
        addSimpleSegment(y, l, r);
    for (auto p : todel)
        segments[y].erase(p);
    segments[y].insert(make_pair(newl, newr));
}

int getAns(int l, int r) {
    return tree.getAns(l, r);
}


int main(int,  char **) {
#ifdef LOCAL
//	freopen("input.txt", "r", stdin);
//	freopen("output.txt", "w", stdout);
#endif

    // TODO(merkurev): change to pseudo online

    for (int i = 0; i < N; i++)
        tree.node[1].setWhiteIfNew(i);
    tree.update(1);

    int q;
    scanf("%d", &q);

    long long sum = 0;
    for (int it = 0; it < q; it++) {
        int t;
        scanf("%d", &t);
        if (t == 1) {
            long long y, l, r;
            scanf("%lld%lld%lld", &y, &l, &r);
            addSegment(y ^ sum, l ^ sum, r ^ sum);
        } else if (t == 2) {
            long long l, r;
            scanf("%lld%lld", &l, &r);
            int ans = getAns(l ^ sum, r ^ sum);
            printf("%d\n", ans);
            sum += ans;
        }
    }


	return 0;
}


